"""
Flask commands to manage torrents on Put.IO.
"""
import logging
logger = logging.getLogger(__name__)

import click
import os
import putiopy
import subprocess

from fswatch import Monitor
from putio_automator.cli import cli
from putio_automator.db import with_db


@cli.group()
def torrents():
    pass

@torrents.command()
@click.pass_context
def add(ctx, parent_id=None):
    "Add a torrent"
    if parent_id == None:
        parent_id = ctx.obj['ROOT']
    folder = ctx.obj['TORRENTS']
    files = os.listdir(folder)
    files = list(f for f in files if os.path.isfile(os.path.join(folder, f)))

    if len(files):
        def func(connection):
            "Anonymous function"
            conn = connection.cursor()

            for name in files:
                path = os.path.join(folder, name)
                size = os.path.getsize(path)

                conn.execute("select datetime(created_at, 'localtime') from torrents where name = ? and size = ?", (name, size))
                row = conn.fetchone()

                if row is None:
                    try:
                        logger.debug('adding torrent: %s' % path)
                        transfer = ctx.obj['CLIENT'].Transfer.add_torrent(path, parent_id=parent_id)
                        os.unlink(path)
                        logger.info('added transfer: %s - %s' % (transfer.id, name))
                    except:
                        info = sys.exc_info()

                        if info[0] == putiopy.ClientError and info[1].type == 'UnknownError':
                            # Assume it's already added
                            os.unlink(path)
                            logger.warning('deleted torrent, already added : %s' % (name,))
                        else:
                            raise

                    conn.execute('insert into torrents (name, size) values (?, ?)', (name, size))
                    connection.commit()
                else:
                    os.unlink(path)
                    logger.warning('deleted torrent, added at %s : %s' % (row[0], name))

        with_db(func)

@torrents.command()
@click.pass_context
def watch(ctx, parent_id=None, mount=False):
    "Watch a folder for new torrents to add"

    if parent_id is None:
        parent_id = ctx.obj['ROOT']

    if mount and not os.path.exists(ctx.obj['TORRENTS']):
        subprocess.call([
            'mount',
            '-a'
        ])

    ctx.invoke(add, parent_id=parent_id)

    monitor = Monitor()
    monitor.add_path(ctx.obj['TORRENTS'])

    def callback(path, evt_time, flags, flags_num, event_num):
        torrent_path = path.decode()
        logger.debug('adding torrent, received event for: %s' % torrent_path)
        transfer = ctx.obj['CLIENT'].Transfer.add_torrent(path.decode(), parent_id=parent_id)
        os.unlink(torrent_path)
        logger.info('added transfer: %s' % transfer)

    monitor.set_callback(callback)
    monitor.start()



"""
Initialize the application.
"""
import logging
logger = logging.getLogger(__name__)

import appdirs
import click
import datetime
import distutils.dir_util
import os
import putiopy
import sqlite3

APP_NAME = 'putio-automator'
APP_AUTHOR = 'datashaman'
DIRS = appdirs.AppDirs(APP_NAME, APP_AUTHOR)

from .db import create_db, database_path
create_db()

def date_handler(obj):
    "Date handler for JSON serialization"
    if isinstance(obj, datetime.datetime) or isinstance(obj, datetime.date):
        return obj.isoformat()
    else:
        return None

def find_config(verbose=False):
    "Search for config on wellknown paths"
    search_paths = [
        os.path.join(os.getcwd(), 'config.py'),
        os.path.join(DIRS.user_data_dir, 'config.py'),
        os.path.join(DIRS.site_data_dir, 'config.py'),
    ]

    config = None

    for search_path in search_paths:
        message = 'Searching %s' % search_path
        logger.debug(message)
        if verbose:
            click.echo(message)

        if os.path.exists(search_path) and not os.path.isdir(search_path):
            config = search_path
            break

    return config

def echo(level, message):
    log_func = getattr(logger, level)
    log_func(message)
    click.echo(message)



"""
Flask command for managing files on Put.IO.
"""
import click
import json
import os
import shutil
import yaml
 
from putio_automator import date_handler, echo, logger
from putio_automator.cli import cli
from putio_automator.db import with_db


@cli.group()
def files():
    pass

@files.command()
@click.pass_context
def list(ctx, parent_id=None):
    "List files"
    if parent_id == None:
        parent_id = ctx.obj['ROOT']
    files = ctx.obj['CLIENT'].File.list(parent_id)
    click.echo(yaml.dump([vars(f) for f in files]))

@files.command()
@click.pass_context
def download(ctx, limit=None, chunk_size=256, parent_id=None, folder="", ignore_existing=False):
    "Download files"
    if parent_id == None:
        parent_id = ctx.obj['ROOT']
    files = ctx.obj['CLIENT'].File.list(parent_id)

    logger.info('%s files found' % len(files))

    if len(files):
        def func(connection):
            "Anonymous func"
            if limit is not None:
                downloaded = 0

            conn = connection.cursor()

            for current_file in files:
                conn.execute("select datetime(created_at, 'localtime') from downloads where name = ? and size = ?", (current_file.name, current_file.size))
                row = conn.fetchone()

                if row is None or ignore_existing:
                    logger.debug('downloading file: %s' % current_file.name)
                    current_file.download(dest=ctx.obj['INCOMPLETE'], delete_after_download=True, chunk_size=int(chunk_size)*1024)
                    logger.info('downloaded file: %s' % current_file.name)

                    src = os.path.join(ctx.obj['INCOMPLETE'], current_file.name)
                    dest = os.path.join(ctx.obj['DOWNLOADS'], folder)
                    shutil.move(src, dest)

                    conn.execute('insert into downloads (id, name, size) values (?, ?, ?)', (current_file.id, current_file.name, current_file.size))
                    connection.commit()

                    if limit is not None:
                        downloaded = downloaded + 1

                        if downloaded > limit:
                            break
                else:
                    logger.warning('file already downloaded at %s : %s' % (row[0], current_file.name))

        with_db(func)



"""
Flask commands
"""
import putio_automator.commands.account
import putio_automator.commands.config
import putio_automator.commands.db
import putio_automator.commands.files
import putio_automator.commands.torrents
import putio_automator.commands.transfers



import appdirs
import os
import pathlib
import sqlite3

from putio_automator import APP_NAME, APP_AUTHOR


user_data_dir = appdirs.user_data_dir(APP_NAME, APP_AUTHOR)
database_path = os.path.join(user_data_dir, 'downloads.db')

def with_db(func):
    if not os.path.exists(database_path):
        pathlib.Path(user_data_dir).mkdir(parents=True, exist_ok=True)
        open(database_path, 'w').close()

    with sqlite3.connect(database_path) as connection:
        func(connection)

def create_db():
    def func(connection):
        c = connection.cursor()
        c.execute('create table if not exists torrents (name character varying primary key, size integer, created_at timestamp default current_timestamp)')
        c.execute('create table if not exists downloads (id integer primary key, name character varying, size integer, created_at timestamp default current_timestamp)')
    with_db(func)



"""
Flask commands for managing Put.IO account
"""

import click
import yaml
import putiopy
import sys

from putio_automator import date_handler, echo, logger
from putio_automator.cli import cli


@cli.group()
def account():
    pass

@account.command()
@click.pass_context
def info(ctx):
    "Show account info"
    try:
        response = ctx.obj['CLIENT'].Account.info()
    except putiopy.ClientError as exc:
        echo('error', exc.message)
        ctx.exit(1)

    if response['status'] == 'OK':
        echo('info', yaml.dump(response['info']))
    else:
        echo('error', yaml.dump(response))
        ctx.exit(1)



"""
Flask commands for managing config of the application
"""

import appdirs
import click
import json
import os
import stat

from putio_automator import APP_NAME, APP_AUTHOR, date_handler, echo, find_config, logger
from putio_automator.cli import cli


def find_config_dist():
    "Search for the config.py.dist file"
    search_paths = [
        os.path.join(os.getcwd(), 'etc', 'config.py.dist'),
        os.path.join(os.getenv('HOME'), '.local', 'share', APP_NAME, 'config.py.dist'),
        os.path.join('/etc', APP_NAME, 'config.py.dist'),
    ]

    config = None

    for search_path in search_paths:
        if os.path.exists(search_path) and not os.path.isdir(search_path):
            config = search_path
            break

    return config

@cli.group()
def config():
    pass

@config.command()
def init(site=False):
    "Prompt the user for config"

    if site:
        base_dir = appdirs.site_data_dir(APP_NAME, APP_AUTHOR)
    else:
        base_dir = appdirs.user_data_dir(APP_NAME, APP_AUTHOR)

    if not os.path.exists(base_dir):
        os.makedirs(base_dir)

    config_path = os.path.join(base_dir, 'config.py')

    incomplete = os.path.realpath(click.prompt('Incomplete directory', 'incomplete'))
    downloads = os.path.realpath(click.prompt('Downloads directory', 'downloads'))
    torrents = os.path.realpath(click.prompt('Torrents directory', 'torrents'))

    putio_token = click.prompt('OAuth Token')

    config_dist = find_config_dist()
    with open(config_dist, 'r') as source:
        contents = (source.read()
                    .replace("os.getenv('PUTIO_TOKEN')",
                             "os.getenv('PUTIO_TOKEN', '" + putio_token + "')")
                    .replace("/files/downloads", downloads)
                    .replace("/files/incomplete", incomplete)
                    .replace("/files/torrents", torrents))

        with open(config_path, 'w') as destination:
            destination.write(contents)

        os.chmod(config_path, stat.S_IRUSR | stat.S_IWUSR)

    echo('info', 'Config written to %s' % config_path)

@config.command()
def show():
    "Show config filename and current config"
    config_file = find_config(verbose=True)

    if config_file:
        echo('info', 'Found config file at %s' % config_file)
    else:
        echo('error', 'Config file not found')

    # logger.info('Current config:\n%s' % json.dumps(app.config, indent=4, default=date_handler))



"""
Flask commands to manage the download database
"""
import click

from putio_automator import echo, logger
from putio_automator.cli import cli
from putio_automator.db import with_db


@cli.group()
def db():
    pass

@db.command()
@click.argument('name')
def forget(name):
    "Delete records of previous downloads in the database"

    def func(connection):
        "Do the above"
        logger.debug('Delete downloads with name like %s' % name)
        conn = connection.cursor()
        conn.execute('delete from downloads where name like ?', ('%%%s%%' % name,))
        echo('info', 'Deleted %d rows with name like %s' % (conn.rowcount, name))

    with_db(func)



"""
Flask commands for managing transfers on Put.IO.
"""
import click
import yaml

from putio_automator import date_handler
from putio_automator.cli import cli


@cli.group()
def transfers():
    pass

@transfers.command()
@click.pass_context
def cancel_by_status(ctx, statuses):
    "Cancel transfers by status"

    if isinstance(statuses, str):
        statuses = statuses.split(',')

    transfer_ids = []
    for transfer in ctx.obj['CLIENT'].Transfer.list():
        if transfer.status in statuses:
            transfer_ids.append(transfer.id)

    if len(transfer_ids):
        ctx.obj['CLIENT'].Transfer.cancel_multi(transfer_ids)

@transfers.command()
def cancel_completed():
    "Cancel completed transfers"
    cancel_by_status('COMPLETED')

@transfers.command()
def cancel_seeding():
    "Cancel seeding transfers"
    cancel_by_status('SEEDING')

@transfers.command()
@click.pass_context
def clean(ctx):
    "Clean finished transfers"
    ctx.obj['CLIENT'].Transfer.clean()

@transfers.command()
@click.pass_context
def groom(ctx):
    "Cancel seeding and completed transfers, and clean afterwards"
    ctx.invoke(cancel_by_status, ['SEEDING', 'COMPLETED'])
    ctx.invoke(clean)

@transfers.command()
@click.pass_context
def list(ctx):
    "List transfers"
    transfers = ctx.obj['CLIENT'].Transfer.list()
    click.echo(yaml.dump([vars(t) for t in transfers]))



#!/usr/bin/env python

import unittest

from setuptools import setup

def suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('tests', pattern='*_test.py')
    return test_suite

setup(
    pbr=True,
    test_suite='setup.suite',
    tests_require=['mock'],
    setup_requires=['pbr>=1.9', 'setuptools>=17.1']
)



#!/usr/bin/env python
"""
Flask script aka CLI entry module.
"""

import click
import logging
import os
import putiopy

from dotenv import load_dotenv

dotenv_path = os.path.join(os.getcwd(), '.env')
if os.path.isfile(dotenv_path):
    load_dotenv(dotenv_path)

@click.group()
@click.option('--token')
@click.option('--root')
@click.option('--downloads')
@click.option('--incomplete')
@click.option('--torrents')
@click.pass_context
def cli(ctx, token=None, root=0, downloads='downloads', incomplete='incomplete', torrents='torrents', log_filename='putio.log', log_level='WARNING'):
    ctx.ensure_object(dict)
    ctx.obj['CLIENT'] = putiopy.Client(token, use_retry=True)
    ctx.obj['ROOT'] = root
    ctx.obj['DOWNLOADS'] = downloads
    ctx.obj['INCOMPLETE'] = incomplete
    ctx.obj['TORRENTS'] = torrents

    logging.basicConfig(filename=log_filename, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s', level=log_level)

from . import commands

def main():
    cli(auto_envvar_prefix='PUTIO', obj={})
